@@ -18,7 +18,7 @@ import os
 import pytest
 
 from mapproxy.config import base_config
-from mapproxy import srs
+from mapproxy import srs, proj
 from mapproxy.srs import SRS, PreferredSrcSRS, SupportedSRS
 
 
@@ -59,7 +59,8 @@ class TestSRS(object):
         srs2 = SRS(srs1)
         assert srs1 == srs2
 
-
+# proj_data_dir test relies on old Proj4 epsg files.
+@pytest.mark.skipif(not proj.USE_PROJ4_API, reason="only for old proj4 lib")
 class Test_0_ProjDefaultDataPath(object):
 
     def test_known_srs(self):
@@ -83,6 +84,7 @@ def custom_proj_data_dir():
     srs.set_datapath(None)
     base_config().srs.proj_data_dir = None
 
+@pytest.mark.skipif(not proj.USE_PROJ4_API, reason="only for old proj4 lib")
 @pytest.mark.usefixtures("custom_proj_data_dir")
 class Test_1_ProjDataPath(object):
 
@@ -154,6 +156,10 @@ class TestSupportedSRS(object):
         assert SRS(4258) not in supported
         assert SRS(25832) in supported
 
+    def test_iter(self, preferred):
+        supported = SupportedSRS([SRS(4326), SRS(25832)], preferred)
+        assert [SRS(4326), SRS(25832)] == [srs for srs in supported]
+
     def test_best_srs(self, preferred):
         supported = SupportedSRS([SRS(4326), SRS(25832)], preferred)
         assert supported.best_srs(SRS(4326)) == SRS(4326)
